# generate test cases for 65Org16b macros

local outFile

local accNames = "ABCDEFGHIJKLMNOQ"

local arithLitOps = "($1000,X) $2000 #$1234 $30000000 ($4000),Y \
	($5000),W $6000,X $70000000,Y $80000000,W $90000000,X"

local arithBakOps = "(bak1,X) bak2 #bak3 bak4 (bak5),Y \
	(bak6),W bak7,X bak8,Y bak9,W bak10,X"

local arithFwdOps = "(fwd1,X) fwd2 #fwd3 fwd4 (fwd5),Y \
	(fwd6),W fwd7,X fwd8,Y fwd9,W fwd10,X"

local shiftOps = "bak2 $2000 fwd2 bak4 $30000000 fwd4 \
	bak7,x $6000,x fwd7,x bak10,x $90000000,x fwd10,x"

local bitOps = "bak2 $2000 fwd2 bak4 $30000000 fwd4"

# -----------------------------------

# set output file name

local function set_outfile(mnemonic) { outFile = "test_" mnemonic ".a" }

# print a line to output file

local function write(this) { print this > outFile }

# print a newline

local function write_nl() { write( "" ) }

# write instruction

local function write_instruc(mnemonic, operand) {

	write( "\t" mnemonic "\t" operand )
}


# write instruction group

local function write_arith(instruc, reg0, reg1, ops) {

	local i
	local cnt
	local operand

	if ( reg0 )
		reg0 = reg0 ","
	if ( reg1 )
		reg1 = reg1 ","		

	cnt = split( ops, operand, " " )
	for ( i = 1; i <= cnt; i++ )
		write_instruc( instruc, reg0 reg1 operand[i] )
	write_nl()
}


# generate arithmetic/logical tests

local function gen_arith(instruc, src) {

	local i
	local sw
	local dst

	for ( i = 1; i <= 16; i++ ) {
		dst = substr( accNames, i, 1 )
		sw = i % 3
		if ( sw == 1 )
			ops = arithLitOps
		else if ( sw == 2 )
			ops = arithBakOps
		else
			ops = arithFwdOps
		write_arith( instruc, dst, src, ops )
	}
}

# generate two-accumulator arithmetic/logical tests

local function gen_arith2(instruc) {

	local i
	local src
	
	for ( i = 1; i <= 16; i++ ) {
		src = substr( accNames, i, 1 )
		gen_arith( instruc, src)
	}
}

# ADC, AND, EOR, ORA, SBC


local function test_arith1(mnemonic) {

	local tag

	tag = "; " mnemonic " - "

	write_nl()

	write( tag "default "A" accumulator" )
	write_nl()
	write_arith( mnemonic, "", "", arithLitOps )

	write( tag "one explicit accumulator" )
	write_nl()

	gen_arith( mnemonic, "" )

	write( tag "two explicit accumulators" )
	write_nl()

	gen_arith2( mnemonic )
	
}

# CMP, LDA, STA

local function test_arith2(mnemonic) {

	local tag

	set_outfile( mnemonic )

	tag = "; " mnemonic " - "

	write_nl()

	write( tag "default "A" accumulator" )
	write_nl()
	write_arith( mnemonic, "", "", arithLitOps )

	write( tag "one explicit accumulator" )
	write_nl()

	gen_arith( mnemonic, "" )
}

# write shift instruction group

local function write_shift_group(mnemonic) {

	local i, j
	local cnt
	local tag
	local src, dst
	local operand

	tag = "; " mnemonic " - "

	write_nl()
	write( tag "default \"A\" accumulator" )
	write_nl()
	write_instruc( mnemonic, "" )

	write_nl()
	write( tag "one explicit accumulator" )
	write_nl()
	for ( i = 1; i <= 4; i++ ) {
		src = substr( accNames, i, 1 )
		write_instruc( mnemonic, src )
	}

	write_nl()
	write( tag "two explicit accumulators" )
	write_nl()
	for ( i = 1; i <= 4; i++ ) {
		dst = substr( accNames, i, 1 )
		for ( j = 1; j <= 4; j++ ) {
			src = substr( accNames, j, 1 )
			write_instruc( mnemonic, dst "," src )
		}
	}

	write_nl()
	write( tag "memory" )
	write_nl()
	cnt = split( shiftOps, operand, " " )
	for ( i = 1; i <= cnt; i++ )
		write_instruc( mnemonic, operand[i] )
	write_nl()			
}

# ASL, LSR, ROL, ROR

local function test_shift(mnemonic) {

	local i

	write_shift_group( mnemonic )
	for ( i = 1; i <= 16; i++ )
		write_shift_group( mnemonic i )
}

# BIT

local function test_bit() {

	local i, j
	local tag
	local cnt
	local operand

	cnt = split( bitOps, operand, " " )

	set_outfile( "BIT" )

	tag = "; BIT - "

	write_nl()
	write( tag "default \"A\" accumulator" )
	write_nl()
	for ( i = 1; i <= cnt; i++ )
		write_instruc( "BIT", operand[i] )

	write_nl()			
	write( tag "explicit accumulator" )
	write_nl()

	for ( i = 1; i<= 16; i++ ) {
		acc = substr( accNames, i, 1 )
		for ( j = 1; j <= cnt; j++ )
			write_instruc( "BIT", acc "," operand[j] )
		write_nl()
	}

}

# DEC, INC (aka DEA, INA)

local function test_dec(mnemonic) {

	local i
	local cnt
	local tag, alt
	local src
	local operand

	set_outfile( mnemonic )

	tag = "; " mnemonic " - "

	write_nl()
	write( tag "default \"A\" accumulator" )
	write_nl()
	write_instruc( mnemonic, "" )

	write_nl()
	write( tag "explicit accumulator" )
	write_nl()
	for ( i = 1; i <= 16; i++ ) {
		src = substr( accNames, i, 1 )
		write_instruc( mnemonic, src )
	}

	write_nl()
	write( tag "memory" )
	write_nl()
	cnt = split( shiftOps, operand, " " )
	for ( i = 1; i <= cnt; i++ )
		write_instruc( mnemonic, operand[i] )
	write_nl()

	alt = substr( mnemonic, 1, 2 ) "A"

	tag = "; alternate " alt " mnemonic - "

	write_nl()
	write( tag "default \"A\" accumulator" )
	write_nl()
	write_instruc( alt, "" )

	write_nl()
	write( tag "explicit accumulator" )
	write_nl()
	for ( i = 1; i <= 16; i++ ) {
		src = substr( accNames, i, 1 )
		write_instruc( alt, src )
	}
}

# write transfer between index register and accumulator instructions

local function write_transfer(mnemonic) {

	local i
	local acc

	tag = "; " mnemonic " - "

	write_nl()
	write( tag "default \"A\" accumulator" )
	write_nl()
	write_instruc( mnemonic, "" )

	write_nl()
	write( tag "explicit accumulator" )
	write_nl()
	for ( i = 1; i <= 16; i++ ) {
		acc = substr( accNames, i, 1 )
		write_instruc( mnemonic, acc )
	}
}

# TAA, TAX, TAY, TAW, TXA, TYA, TWA

local function test_taa() {

	local i, j
	local src, dst

	set_outfile( "TAA" )

	tag = "; TAA - "
	write_nl()
	write( tag "default \"A\" accumulator" )
	write_nl()
	write_instruc( "TAA", "" )

	write_nl()
	write( tag "two explicit accumulators" )
	write_nl()
	for ( i = 1; i <= 16; i++ ) {
		src = substr( accNames, i, 1 )
		for ( j = 1; j <= 16; j++ ) {
			dst = substr( accNames, j, 1 )
			write_instruc( "TAA", src "," dst )
		}
		write_nl()
	}
	
	write_transfer( "TAX" )
	write_transfer( "TAY" )
	write_transfer( "TAW" )

	write_transfer( "TXA" )
	write_transfer( "TYA" )
	write_transfer( "TWA" )

	write_transfer( "TAZ" )
	write_transfer( "TAS" )
	
	write_transfer( "TZA" )
	write_transfer( "TSA" )	
}

BEGIN {

	test_taa()
}